# Wagtail 4.0 release notes - IN DEVELOPMENT

```{contents}
---
local:
depth: 1
---
```

## What's new

### Image renditions can now be prefetched by filter

When using a queryset to render a list of images, you can now use the ``prefetch_renditions()`` queryset method to prefetch the renditions needed for rendering with a single extra query, similar to ``prefetch_related``. If you have many renditions per image, you can also call it with filters as arguments - ``prefetch_renditions("fill-700x586", "min-600x400")`` - to fetch only the renditions you intend on using for a smaller query. For long lists of images, this can provide a significant boost to performance. See [](prefetching_image_renditions) for more examples. This feature was developed by Tidiane Dia and Karl Hobley.

### Other features

 * Add clarity to confirmation when being asked to convert an external link to an internal one (Thijs Kramer)
 * Convert various pages in the documentation to Markdown (Khanh Hoang, Vu Pham, Daniel Kirkham, LB (Ben) Johnston, Thiago Costa de Souza, Benedict Faw)
 * Add `base_url_path` to `ModelAdmin` so that the default URL structure of app_label/model_name can be overridden (Vu Pham, Khanh Hoang)
 * Add `full_url` to the API output of `ImageRenditionField` (Paarth Agarwal)
 * Use `InlinePanel`'s label when available for field comparison label (Sandil Ranasinghe)
 * Drop support for Safari 13 by removing left/right positioning in favour of CSS logical properties (Thibaud Colas)
 * Use `FormData` instead of jQuery's `form.serialize` when editing documents or images just added so that additional fields can be better supported (Stefan Hammer)
 * Add informational Codecov status checks for GitHub CI pipelines (Tom Hu)
 * Make it possible to reuse and customise Wagtail’s fonts with CSS variables (LB (Ben) Johnston)
 * Add better handling and informative developer errors for cross linking URLS (e.g. success after add) in generic views `wagtail.admin.views.generic` (Matt Westcott)
 * Introduce `wagtail.admin.widgets.chooser.BaseChooser` to make it easier to build custom chooser inputs (Matt Westcott)
 * Introduce JavaScript chooser module, including a SearchController class which encapsulates the standard pattern of re-rendering the results panel in response to search queries and pagination (Matt Westcott)
 * Add ability to select multiple items at once within bulk actions selections when holding shift on subsequent clicks (Hitansh Shah)
 * Upgrade notification, shown to admins on the dashboard if Wagtail is out of date, will now link to the release notes for the closest minor branch instead of the latest patch (Tibor Leupold)
 * Upgrade notification can now be configured to only show updates when there is a new LTS available via `WAGTAIL_ENABLE_UPDATE_CHECK = 'lts'` (Tibor Leupold)
 * Implement redesign of the Workflow Status dialog, fixing accessibility issues (Steven Steinwand)
 * Add the ability to change the number of images displayed per page in the image library (Tidiane Dia, with sponsorship from YouGov)
 * Allow users to sort by different fields in the image library (Tidiane Dia, with sponsorship from YouGov)
 * Add `prefetch_renditions` method to `ImageQueryset` for performance optimisation on image listings (Tidiane Dia, Karl Hobley)
 * Add ability to define a custom `get_field_clean_name` method when defining `FormField` models that extend `AbstractFormField` (LB (Ben) Johnston)
 * Migrate Home (Dashboard) view to use generic Wagtail class based view (LB (Ben) Johnston)
 * Combine most of Wagtail’s stylesheets into the global `core.css` file (Thibaud Colas)
 * Add new Breadcrumbs to the Wagtail pattern library (Paarth Agarwal)
 * Adopt new Page Editor UI tabs in the workflow history report page (Paarth Agarwal)
 * Update `ReportView` to extend from generic `wagtail.admin.views.generic.models.IndexView` (Sage Abdullah)

### Bug fixes

 * Fix issue where `ModelAdmin` index listings with export list enabled would show buttons with an incorrect layout (Josh Woodcock)
 * Fix typo in `ResumeWorkflowActionFormatter` message (Stefan Hammer)
 * Throw a meaningful error when saving an image to an unrecognised image format (Christian Franke)
 * Remove extra padding for headers with breadcrumbs on mobile viewport (Steven Steinwand)
 * Replace `PageRevision` with generic `Revision` model (Sage Abdullah)
 * Ensure the upgrade notification request for the latest release, which can be disabled via the `WAGTAIL_ENABLE_UPDATE_CHECK` sends the referrer origin with `strict-origin-when-cross-origin` (Karl Hobley)
 * Ensure radio buttons / checkboxes display vertically under Django 4.0 (Matt Westcott)
 * Ensure that custom document or image models support custom tag models (Matt Westcott)
 * Ensure comments use translated values for their placeholder text (Stefan Hammer)
 * Ensure the upgrade notification, shown to admins on the dashboard if Wagtail is out of date, content is translatable (LB (Ben) Johnston)
 * Only show the re-ordering option to users that have permission to publish pages within the page listing (Stefan Hammer)
 * Ensure `TabbedInterface` will not show a tab if no panels are visible due to permissions (Paarth Agarwal)


## Upgrade considerations

### `base_url_path` keyword argument added to AdminURLHelper

The `wagtail.contrib.modeladmin.helpers.AdminURLHelper` class now accepts a `base_url_path` keyword argument on its constructor. Custom subclasses of this class should be updated to accept this keyword argument.

### Dropped support for Safari 13

Safari 13 will no longer be officially supported as of this release, this deviates the current support for the last 3 version of Safari by a few months and was required to add better support for RTL languages.

### `PageRevision` replaced with `Revision`

The `PageRevision` model has been replaced with a generic `Revision` model. If you use the `PageRevision` model in your code, make sure that:

 * Creation of `PageRevision` objects should be updated to create `Revision` objects using the page's `id` as the `object_id`, the default `Page` model's content type as the `base_content_type`, and the page's specific content type as the `content_type`.
 * Queries that use the `PageRevision.objects` manager should be updated to use the `Revision.page_revisions` manager.
 * `Revision` queries that use `Page.id` should be updated to cast the `Page.id` to a string before using it in the query (e.g. by using `str()` or `Cast("page_id", output_field=CharField())`).
 * `Page` queries that use `PageRevision.page_id` should be updated to cast the `Revision.object_id` to an integer before using it in the query (e.g. by using `int()` or `Cast("object_id", output_field=IntegerField())`).
 * Access to `PageRevision.page` should be updated to `Revision.content_object`.

If you maintain a package across multiple Wagtail versions that includes a model with a `ForeignKey` to the `PageRevision` model, you can create a helper function to correctly resolve the model depending on the installed Wagtail version, for example:

```python
from django.db import models
from wagtail import VERSION as WAGTAIL_VERSION


def get_revision_model():
    if WAGTAIL_VERSION >= (4, 0):
        return "wagtailcore.Revision"
    return "wagtailcore.PageRevision"


class MyModel(models.Model):
    # Before
    # revision = models.ForeignKey("wagtailcore.PageRevision")
    revision = models.ForeignKey(get_revision_model(), on_delete=models.CASCADE)
```


### `AdminChooser` replaced with `BaseChooser`

Custom choosers should no longer use `wagtail.admin.widgets.chooser.AdminChooser` which has been replaced with `wagtail.admin.widgets.chooser.BaseChooser`.
